package EFS::AFS::Util;

use strict;
use warnings;
use English;

use File::Basename qw( dirname basename );
use Params::Validate qw( :all );
use Try::Tiny;

sub create_volume {

    my $class = shift;

    my (%args) = validate( @_, {
        name       => { type => SCALAR },
        cell       => { type => SCALAR },
        partitions => { type => ARRAYREF },
        readonly   => { type => SCALAR,  optional => 1 },
        acl        => { type => HASHREF, optional => 1 },
    });

    my $name       = $args{name};
    my $cell       = $args{cell};
    my $partitions = $args{partitions};

    my $acl = $args{acl} || {
        q{system:anyuser} => q{rl},
    };

    my $vos = AFS::Command::VOS->new;

    my $result = $vos->listvldb( name => $name, cell => $cell );

    if ( not $result ) {
        my ($server,$partition) = @{ $partitions->[0] };
        $vos->create(
            name      => $name,
            cell      => $cell,
            server    => $server,
            partition => $partition,
        );
        STDOUT->print(
            qq{Created AFS volume $name in $cell on $server:$partition\n}
        );
    };

    my $path = qq{/afs/.:mount/$cell:$name};

    $class->set_acl( path => $path, acl => $acl );

    EFS::Util->Chmod( path => $path, mode => 0755 );

    return 1 if not $args{readonly};

    $result = $vos->listvldb( name => $name, cell => $cell );

    my $vldbentry = $result->getVLDBEntry( name => $name );

    my $rwserver    = q{};
    my $rwpartition = q{};
    my %roserver    = ();

    foreach my $vldbsite ( $vldbentry->getVLDBSites ) {
        if ( $vldbsite->type eq q{RW} ) {
            $rwserver    = $vldbsite->server;
            $rwpartition = $vldbsite->partition;
        } else {
            $roserver{ $vldbsite->server } = $vldbsite->partition;
        }
    }

    my %newserver = ();

    foreach my $serverpart ( @{ $partitions } ) {
        my ($server,$partition) = @{ $serverpart };
        $newserver{ $server } = $partition;
    }

    my $rohave = scalar keys %roserver;
    my $rowant = scalar keys %newserver;

    return 1 if $rohave >= $rowant;

    if ( not $roserver{ $rwserver } ) {
        $vos->addsite(
            id        => $name,
            cell      => $cell,
            server    => $rwserver,
            partition => $rwpartition,
        );
        STDOUT->print(
            qq{Added RO clone site for $name in $cell in $rwserver:$rwpartition\n}
        );
        $rohave++;
    }

    while ( $rohave < $rowant ) {

        my ($roserver) = ( grep { not $roserver{$_} } keys %newserver ) or
            die qq{Unable to determine next RO site for $name in $cell\n};

        my $ropartition = $newserver{ $roserver };

        $vos->addsite(
            id        => $name,
            cell      => $cell,
            server    => $roserver,
            partition => $ropartition,
        );

        STDOUT->print(
            qq{Added RO site for $name in $cell in $roserver:$ropartition\n}
        );

        $rohave++;

    }

    $vos->release( id => $name, cell => $cell );
    STDOUT->print( qq{Released AFS volume $name in $cell\n} );
    return 1;

}

sub create_mount {

    my $class = shift;

    my (%args) = validate( @_, {
        volume => { type => SCALAR },
        parent => { type => SCALAR },
        path   => { type => SCALAR },
        cell   => { type => SCALAR },
    });

    my $volume = $args{volume};
    my $parent = $args{parent};
    my $path   = $args{path};
    my $cell   = $args{cell};

    my $vos = AFS::Command::VOS->new;
    my $fs  = AFS::Command::FS->new;

    my $vldbentry = $vos->getVLDBEntry( name => $parent, cell => $cell );
    my $readonly  = $vldbentry->sites > 1;

    my $romount = qq{/afs/.:mount/$cell:$parent.readonly/$path};
    my $rwmount = qq{/afs/.:mount/$cell:$parent/$path};

    if ( $readonly ) {
        my $roresult = $fs->lsmount( dir => $romount );
        my $ropath   = $roresult->getPath( $romount );
        if ( not $ropath->error and $ropath->volname eq $volume ) {
            STDOUT->print( qq{Volume $volume is already mounted on $parent:$path in $cell\n} );
            return 1;
        }
    }

    my $rwresult = $fs->lsmount( dir => $rwmount );
    my $rwpath   = $rwresult->getPath( $rwmount );

    if ( not $rwpath->error ) {
        if ( $rwpath->volname ne $volume ) {
            die(
                qq{Incorrect volume mounted on $rwpath:\n},
                qq{Have => }, $rwpath->volname, q{\n},
                qq{Want => $volume\n},
            );
        }
    } else {

        $fs->setacl(
            dir => dirname( $rwmount ),
            acl => { q{system:administrators} => q{i} },
        );

        $fs->mkmount( dir => $rwmount, vol => $volume );
        STDOUT->print( qq{Created AFS mount for $volume on $path in $cell\n} );

        $fs->setacl(
            dir => dirname( $rwmount ),
            acl => { q{system:administrators} => q{none} },
        );

    }

    if ( $readonly ) {
        $vos->release( id => $parent, cell => $cell );
        STDOUT->print( qq{Released AFS volume $parent in $cell\n} );
    }

    return 1;
    
}

sub remove_mount {

    my $class = shift;

    my (%args) = validate( @_, {
        volume   => { type => SCALAR },
        parent   => { type => SCALAR },
        path     => { type => SCALAR },
        cell     => { type => SCALAR },
    });

    my $volume = $args{volume};
    my $parent = $args{parent};
    my $path   = $args{path};
    my $cell   = $args{cell};

    my $vos = AFS::Command::VOS->new;
    my $fs  = AFS::Command::FS->new;

    my $vldbentry = $vos->getVLDBEntry( name => $parent, cell => $cell );

    if ( not $vldbentry ) {
        # XXX: Fatal?
        STDOUT->print( qq{No such parent volume $parent in $cell\n} );
        return 1;
    }

    my $readonly  = $vldbentry->sites > 1;

    my $romount = qq{/afs/.:mount/$cell:$parent.readonly/$path};
    my $rwmount = qq{/afs/.:mount/$cell:$parent/$path};

    my $roresult = $fs->lsmount( dir => $romount );
    my $ropath   = $roresult->getPath( $romount );

    if ( $ropath->error and $ropath->error =~ m{doesn.t exist}ms ) {
        STDOUT->print( qq{Volume $volume is already unmounted from $parent.readonly:$path in $cell\n} );
        return 1;
    }

    my $rwresult = $fs->lsmount( dir => $rwmount );
    my $rwpath   = $rwresult->getPath( $rwmount );

    if ( $rwpath->error and $rwpath->error =~ m{doesn.t exist}ms ) {
        STDOUT->print( qq{Volume $volume is already unmounted from $parent:$path in $cell\n} );
    } else {

        $fs->setacl(
            dir => dirname( $rwmount ),
            acl => { q{system:administrators} => q{d} },
        );

        $fs->rmmount( dir => $rwmount );
        STDOUT->print( qq{Removed AFS mount for $volume on $path in $cell\n} );

        $fs->setacl(
            dir => dirname( $rwmount ),
            acl => { q{system:administrators} => q{none} },
        );

    }

    if ( $readonly ) {
        $vos->release( id => $parent, cell => $cell );
        STDOUT->print( qq{Released AFS volume $parent in $cell\n} );
    }

    return 1;
    
}

sub remove_volume {

    my $class = shift;

    my (%args) = validate( @_, {
        name => { type => SCALAR },
        cell => { type => SCALAR },
    });

    my $name = $args{name};
    my $cell = $args{cell};

    my $vos = AFS::Command::VOS->new;

    my $result = $vos->listvldb( name => $name, cell => $cell );

    if ( not $result ) {
        STDOUT->print( qq{Volume $name has already been removed from $cell\n} );
        return 1;
    }

    my $vldbentry = $result->getVLDBEntry( name => $name );
    my $readonly  = $vldbentry->sites > 1;

    foreach my $type ( qw( RO RW ) ) {
        foreach my $vldbsite ( $vldbentry->getVLDBSites ) {
            next if $vldbsite->type ne $type;
            my $volume    = $type eq q{RO} ? qq{$name.readonly} : $name;
            my $server    = $vldbsite->server;
            my $partition = $vldbsite->partition;
            $vos->remove(
                id        => $volume,
                server    => $server,
                partition => $partition,
                cell      => $cell,
            );
            STDOUT->print( qq{Removed $volume from $server:$partition in $cell\n} );
        }
    }

    return 1;

}

sub set_acl {

    my $class = shift;

    my (%args) = validate( @_, {
        path => { type => SCALAR },
        acl  => { type => HASHREF },
    });

    my $path = $args{path};

    my $fs = AFS::Command::FS->new;

    my $result = $fs->getPathInfo(
        method => q{listacl},
        path   => $path,
    );

    my $haveacl = $result->getACLNormal;
    my $wantacl = $args{acl};
    my $needacl = {};

    $wantacl->{ q{system:administrators} } ||= q{none};

    foreach my $principal ( $haveacl->getPrincipals ) {
        my $rights = $haveacl->getRights( $principal );
        if ( not $wantacl->{$principal} or $wantacl->{$principal} eq q{none} ) {
            $needacl->{$principal} = q{none};
        } elsif ( $wantacl->{$principal} ne $rights ) {
            $needacl->{$principal} = $wantacl->{$principal};
        }
    }

    foreach my $principal ( keys %{ $wantacl } ) {
        if ( not $haveacl->getRights( $principal ) and $wantacl->{$principal} ne q{none} ) {
            $needacl->{$principal} = $wantacl->{$principal};
        }
    }

    if ( not keys %{ $needacl } ) {
        STDOUT->print( qq{No changes to ACL on $path\n} );
        return 1;
    }

    foreach my $principal ( sort keys %{ $needacl } ) {
        my $rights = $needacl->{$principal};
        STDOUT->print( qq{Changing ACL entry on $path for $principal to $rights\n} );
    }

    return $fs->setacl( dir => $path, acl => $needacl );

}

sub create_symlink {

    my $class = shift;

    my (%args) = validate( @_, {
        volume   => { type => SCALAR },
        cell     => { type => SCALAR },
        path     => { type => SCALAR },
        target   => { type => SCALAR },
    });

    my $volume = $args{volume};
    my $cell   = $args{cell};
    my $path   = $args{path};
    my $target = $args{target};

    my $vos = AFS::Command::VOS->new;
    my $fs  = AFS::Command::FS->new;

    my $vldbentry = $vos->getVLDBEntry( name => $volume, cell => $cell );
    my $readonly  = $vldbentry->sites > 1;

    my $romount = qq{/afs/.:mount/$cell:$volume.readonly};
    my $rwmount = qq{/afs/.:mount/$cell:$volume};

    my $checkpath = $readonly ? qq{$romount/$path} : qq{$rwmount/$path};

    if ( -l $checkpath ) {
        my $checktarget = readlink $checkpath or
            die qq{Unable to readlink $checkpath: $ERRNO\n};
        if ( $checktarget eq $target ) {
            # XXX: EFS::Util->MakeLink is silent in this case, too.
            return 1;
        }
    }
    
    $class->_create_dirs(
        top    => $rwmount,
        bottom => dirname( qq{$rwmount/$path} ),
    );

    $fs->setacl(
        dir => dirname( qq{$rwmount/$path} ),
        acl => { q{system:administrators} => q{i} },
    );
    EFS::Util->MakeLink( from => qq{$rwmount/$path}, to => $target );
    $fs->setacl(
        dir => dirname( qq{$rwmount/$path} ),
        acl => { q{system:administrators} => q{none} },
    );

    if ( $readonly ) {
        $vos->release( id => $volume, cell => $cell );
        STDOUT->print( qq{Released AFS volume $volume in $cell\n} );
    }

    return 1;

}

sub remove_symlink {

    my $class = shift;

    my (%args) = validate( @_, {
        volume => { type => SCALAR },
        cell   => { type => SCALAR },
        path   => { type => SCALAR },
    });

    my $volume = $args{volume};
    my $cell   = $args{cell};
    my $path   = $args{path};

    my $vos = AFS::Command::VOS->new;
    my $fs  = AFS::Command::FS->new;

    my $vldbentry = $vos->getVLDBEntry( name => $volume, cell => $cell );
    my $readonly  = $vldbentry->sites > 1;

    my $romount = qq{/afs/.:mount/$cell:$volume.readonly};
    my $rwmount = qq{/afs/.:mount/$cell:$volume};

    if ( not -l qq{$romount/$path} and not -l qq{$rwmount/$path} ) {
        return 1;
    }

    if ( -l qq{$rwmount/$path} ) {

        $fs->setacl(
            dir => dirname( qq{$rwmount/$path} ),
            acl => { q{system:administrators} => q{d} },
        );

        unlink qq{$rwmount/$path} or
            die qq{Unable to unlink $rwmount/$path: $ERRNO\n};

        STDOUT->print( qq{Removed symlink $rwmount/$path\n} );

        $fs->setacl(
            dir => dirname( qq{$rwmount/$path} ),
            acl => { q{system:administrators} => q{none} },
        );

    }

    if ( $readonly ) {
        $vos->release( id => $volume, cell => $cell );
        STDOUT->print( qq{Released AFS volume $volume in $cell\n} );
    }

    return 1;

}

sub _create_dirs {

    my $class = shift;

    my (%args) = validate( @_, {
        top    => { type => SCALAR },
        bottom => { type => SCALAR },
    });

    my $top    = $args{top};
    my $bottom = $args{bottom};

    my $fs = AFS::Command::FS->new;

    my $dir = $bottom;
    my @dirs = ();

    while ( $dir ne $top ) {
        unshift @dirs, $dir;
        $dir = dirname $dir;
    }

    foreach my $dir ( sort @dirs ) {
        next if -d $dir;
        $fs->setacl(
            dir => dirname( $dir ),
            acl => { q{system:administrators} => q{i} },
        );
        EFS::Util->MakeDir( path => $dir );
        $fs->setacl(
            dir => dirname( $dir ),
            acl => { q{system:administrators} => q{none} },
        );
    }

    return 1;

}

1;
