package EFS::CLI::Metaproj;

use strict;
use warnings;
use English;

use feature q{switch};

use base q{EFS::CLI};
use base q{EFS::DBI};
use base q{EFS::DistableObject};

use Params::Validate qw( :all );

sub define {
    die(
        qq{'efs define metaproj' has been deprecated\n},
        qq{Use 'efs create metaproj' instead\n},
    );
    return;
}

sub undef {
    die(
        qq{'efs undef metaproj' has been deprecated\n},
        qq{Use 'efs destroy metaproj' instead\n},
    );
    return;
}

sub create {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        %EFS::CLI::validate_dist_args_common,
        name        => { type => SCALAR },
        username    => { type => SCALAR },
        groupname   => { type => SCALAR },
        owner_list  => { type => SCALAR, optional => 1 },
        notify_list => { type => SCALAR, optional => 1 },
    });

    my $username  = EFS::Username->new({ name => $args{username} });
    my $groupname = EFS::Groupname->new({ name => $args{groupname} });

    my $resultset = $class->get_resultset;

    my ($metaproj) = $resultset->search({ name => $args{name} });

    if ( $metaproj ) {

        if ( $metaproj->username ne $username->name ) {
            $metaproj->set_column( username => $username->name );
            $metaproj->update;
            STDOUT->print(
                qq{Updated metaproj '$metaproj', },
                qq{set username to '$username'\n}
            );
        }

        if ( $metaproj->groupname ne $groupname->name ) {
            $metaproj->set_column( groupname => $groupname->name );
            $metaproj->update;
            STDOUT->print(
                qq{Updated metaproj '$metaproj', },
                qq{set groupname to '$groupname'\n}
            );
        }

        if ( $args{owner_list} ) {
            if ( $metaproj->owner_list ne $args{owner_list} ) {
                $metaproj->set_column( owner_list => $args{owner_list} );
                $metaproj->update;
                STDOUT->print(
                    qq{Updated metaproj '$metaproj', },
                    qq{set owner_list to '$args{owner_list}'\n}
                );
            }
        }

        if ( $args{notify_list} ) {
            if ( $metaproj->notify_list ne $args{notify_list} ) {
                $metaproj->set_column( notify_list => $args{notify_list} );
                $metaproj->update();
                STDOUT->print(
                    qq{Updated metaproj '$metaproj', },
                    qq{set notify_list to '$args{notify_list}'\n}
                );
            }
        }

    } else {

        my $parameter_rs = $class->get_resultset( q{Parameter} );

        my $root_domain = $parameter_rs->configured(q{root_domain}) ||
            die qq{No root_domain parameter has been defined.\n};

        $args{owner_list} ||= join q{@}, $username->name, $root_domain;

        $metaproj       = $resultset->create({
            name        => $args{name},
            username    => $username->name,
            groupname   => $groupname->name,
            owner_list  => $args{owner_list},
            notify_list => $args{notify_list},
        });

        STDOUT->print(
            qq{Created metaproj '$metaproj' },
            qq{with username '$username', },
            qq{groupname '$groupname', },
            ( $args{owner_list} ?
                  qq{owner_list '$args{owner_list}'\n} : q{} ),
            ( $args{notify_list} ?
                  qq{notify_list '$args{notify_list}'\n} : q{} ),
        );

    }

    return $metaproj;

}

sub destroy {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR },
    });

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert( name => $args{name} );

    if ( my @projects = $metaproj->projects ) {
        die(
            qq{Unable to destroy metaproj '$metaproj'\n},
            qq{The following projects still exist:\n\t},
            join(qq{\n\t}, map { $_->object_id } @projects ), qq{\n},
        );
    }

    my $cell_rs = $class->get_resultset( q{Cell} );

    my @dev_cells = $cell_rs->search({ celltype => q{dev}, hidden => 0 });

    my @dist_cells = (
        $cell_rs->search( { celltype => q{prod}, hidden => 0 } ),
        @dev_cells,
    );

    my $fm_dist_cells = EFS::ForkManager->new({

        list => \@dist_cells,

        action => sub {

            my $cell = shift;

            EFS::RPC->execute(
                cell   => $cell,
                class  => $class,
                method => q{destroy_cell},
                args   => [
                    cell => $cell->name,
                    name => $metaproj->name,
                    type => q{dist},
                ],
            );

        },

    });

    $fm_dist_cells->run || die(
        qq{Unable to remove /efs/dist/$metaproj from these cells:\n\t},
        join ( qq{\n\t}, $fm_dist_cells->get_failed ), qq{\n},
    );

    my $fm_dev_cells = EFS::ForkManager->new({

        list => \@dev_cells,

        action => sub {

            my $cell = shift;

            EFS::RPC->execute(
                cell   => $cell,
                class  => $class,
                method => q{destroy_cell},
                args   => [
                    cell => $cell->name,
                    name => $metaproj->name,
                    type => q{dev},
                ],
            );

        },

    });

    $fm_dev_cells->run || die(
        qq{Unable to remove /efs/dev/$metaproj from these cells:\n\t},
        join ( qq{\n\t}, $fm_dev_cells->get_failed ), qq{\n},
    );

    my $object_id = $metaproj->object_id;

    $metaproj->grouprights->delete_all;
    $metaproj->userrights->delete_all;
    $metaproj->attrs->delete_all;
    $metaproj->delete;

    STDOUT->print( qq{Destroyed metaproj $object_id\n} );

    return 1;

}

sub destroy_cell {

    my $class = shift;

    my (%args) = validate( @_, {
        name => { type => SCALAR },
        cell => { type => SCALAR },
        type => { type => SCALAR },
    });

    my $resultset = $class->get_resultset;
    my $metaproj = $resultset->retrieve_bynames_assert( name => $args{name} );

    my $cell_rs = $class->get_resultset(q{Cell});
    my $cell = $cell_rs->retrieve_bynames_assert( name => $args{cell}, );

    my $type = $args{type};

    given ( $cell->fstype ) {

        when ( q{afs} ) {

            my $afsvolume = $metaproj->get_afsvolume( $type );

            EFS::AFS::Util->remove_mount(
                volume => $afsvolume->name,
                parent => qq{efs.$type},
                path   => $metaproj->name,
                cell   => $cell->name,
            );
                
            EFS::AFS::Util->remove_volume(
                name => $afsvolume->name,
                cell => $cell->name,
            );

        }

        when ( q{nfs} ) {

            my $admin_path = $cell->admin_path( type => $type );

            my @destroy = glob (qq{$admin_path/*/$type/$metaproj});

            if ( -d qq{$admin_path/$type/$metaproj} ) {
                push @destroy, qq{$admin_path/$type/$metaproj};
            }

            if ( not @destroy ) {
                STDOUT->print( qq{No directory $cell:/efs/$type/$metaproj to destroy\n} );
                return 1;
            }

            foreach my $destroy ( @destroy ) {
                EFS::Util->RemoveDirHierarchy(
                    top   => $destroy,
                    empty => 1,
                );
            }

        }

    }        

    return 1;

}

sub display {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name   => { type => SCALAR, optional => 1 },
        sortby => { type => SCALAR, default  => q{name}, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    if ( not $args{name} ) {
        $class->display_objects(
            objects     => [ $resultset->all ],
            sort_column => $args{sortby},
        );
        return 1;
    }

    my $metaproj = $resultset->retrieve_bynames_assert( name => $args{name} );

    $class->display_objects( objects => [ $metaproj ] );

    my @optional = $metaproj->get_efs_column_group( q{optional} );

    my $feature_rs = $class->get_resultset( q{Feature} );

    if ( not $feature_rs->configured( q{disable_restricted_access} ) ) {
        push @optional, $metaproj->get_efs_column_group( q{access} );
    }

    if ( @optional ) {
        $metaproj->display_options( columns => \@optional );
    }

    if ( my @grouprights = sort $metaproj->grouprights ) {
        $grouprights[0]->display_objects( objects => \@grouprights );
    }

    if ( my @userrights = sort $metaproj->userrights ) {
        $userrights[0]->display_objects( objects => \@userrights );
    }

    if ( my @attrs = sort $metaproj->attrs ) {
        $attrs[0]->display_objects( objects => \@attrs );
    }

    if ( my @projects = sort $metaproj->projects ) {
        $projects[0]->display_objects(
            objects     => \@projects,
            sort_column => $args{sortby},
        );
    }

    return 1;

}

sub history {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR },
        %EFS::CLI::History::validation_specs,
    });

    $args{metaproj} = delete $args{name};

    return EFS::CLI::History->history(%args);

}

sub chown {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name    => { type => SCALAR },
        verbose => { type => BOOLEAN, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert(
        name => $args{name},
    );

    my @projects = $class->sort_objects( $metaproj->projects );

    if ( not @projects ) {
        STDOUT->print(
            qq{There are no projects defined yet for metaproj '$metaproj'\n}
        );
        return 1;
    }

    STDOUT->print( qq{Chowning entire metaproj '$metaproj'\n} );

    $args{metaproj} = delete $args{name};

    my $fm_projects = EFS::ForkManager->new({

        parallel => 1,

        list => \@projects,

        action => sub {

            my $project = shift;

            my $object_id = $project->object_id(q{ });

            return EFS::RPC->execute(
                cell   => $project->cell,
                class  => q{EFS::CLI::Project},
                method => q{chown_rpc},
                args   => [
                    authorized => 1,
                    name       => $project->name,
                    %args,
                ],
                history => {
                    parentid => $args{history},
                    values   => {
                        request  => qq{chown project $object_id},
                        metaproj => $args{metaproj},
                        project  => $project->name,
                    },
                },
            );

        },

    });

    $fm_projects->run || die(
        qq{Unable to chown the following projects:\n\t},
        join(qq{\n\t}, map { $_->object_id } $fm_projects->get_failed ), qq{\n}
    );

    return 1;

}

sub disttree {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR },
        %EFS::CLI::validate_dist_args,
    });

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert(
        name => $args{name},
    );

    my @cells = $class->handle_cell_arguments(
        %args,
        include_parents => 1,
    );

    my @releases       = $metaproj->active_releases;
    my @releasealiases = $metaproj->releasealiases;

    if ( grep { $_->celltype eq q{prod} } @cells ) {
        @releases       = grep { $_->has_only_prod_stages } @releases;
        @releasealiases = grep { $_->has_only_prod_stages } @releasealiases;
    }

    my $tree = EFS::DependencyTree->new({
        seeds => [ @releases, @releasealiases ],
        type  => q{runtime},
    });

    delete $args{name};

    if ( $args{dryrun} ) {
        STDOUT->print( $tree->get_tree_text( cells => \@cells ) );
        return 1;
    } else {
        return $tree->dist( %args );
    }

}

sub dist {
    return die(
        qq{This operation is no longer supported.\n},
        qq{Use 'efs disttree metaproj' instead.\n},
    );
}

sub purge {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR | OBJECT },
        %EFS::CLI::validate_purge_args,
    });

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert(
        name => $args{name},
    );

    my @cells = $class->handle_cell_arguments(
        %args,
        include_parents => 1,
        default_scope   => q{global},
    );

    my @projects = $class->sort_objects( $metaproj->projects );

    if ( not @projects ) {
        STDOUT->print(
            qq{There are no projects defined yet for metaproj '$metaproj'\n}
        );
        return 1;
    }

    STDOUT->print( qq{Purging entire metaproj '$metaproj'\n} );

    $args{metaproj} = delete $args{name};

    my $fm_projects = EFS::ForkManager->new({

        parallel => 1,

        list => \@projects,

        action => sub {

            my $project = shift;

            my $object_id = $project->object_id(q{ });

            return EFS::RPC->execute(
                cell   => $project->cell,
                class  => q{EFS::CLI::Project},
                method => q{purge_rpc},
                args   => [
                    authorized => 1,
                    name       => $project->name,
                    %args,
                ],
                history => {
                    parentid => $args{history},
                    values   => {
                        request  => qq{purge project $object_id},
                        metaproj => $args{metaproj},
                        project  => $project->name,
                    },
                },
            );

        },

    });

    $fm_projects->run || die(
        qq{Unable to purge the following projects:\n\t},
        join( qq{\n\t}, $fm_projects->get_failed ), qq{\n}
    );

    return 1;

}

sub execlinks {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR | OBJECT },
        %EFS::CLI::validate_diststate_args,
    });

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert(
        name => $args{name},
    );

    my @projects = $class->sort_objects( $metaproj->projects );

    if ( not @projects ) {
        STDOUT->print(
            qq{There are no projects defined yet for metaproj '$metaproj'\n}
        );
        return 1;
    }

    my @cells = $class->handle_cell_arguments(
        %args,
        default_scope => q{global},
    );

    STDOUT->print(
        qq{Updating execlinks in entire metaproj '$metaproj'\n}
    );

    $args{metaproj} = delete $args{name};

    my $fm_projects = EFS::ForkManager->new({

        list => \@projects,

        action => sub {

            my $project = shift;

            my $object_id = $project->object_id(q{ });

            return EFS::RPC->execute(
                cell   => $project->cell,
                class  => q{EFS::CLI::Project},
                method => q{execlinks_rpc},
                args   => [
                    authorized => 1,
                    name       => $project->name,
                    %args,
                ],
                history => {
                    parentid => $args{history},
                    values   => {
                        request  => qq{execlinks project $object_id},
                        metaproj => $args{metaproj},
                        project  => $project->name,
                    },
                },
            );

        },

    });

    $fm_projects->run || die(
        qq{Unable to update execlinks in the following projects:\n\t},
        join(qq{\n\t}, map { $_->object_id } $fm_projects->get_failed ), qq{\n}
    );

    STDOUT->print(
        qq{Successfully updated all execlinks for metaproj '$metaproj'\n}
    );

    return 1;

}

sub audit {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name    => { type => SCALAR | OBJECT },
        audits  => { type => SCALAR, default => q{required}, optional => 1 },
        force   => { type => BOOLEAN, optional => 1 },
        verbose => { type => BOOLEAN, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert(
        name => $args{name},
    );

    my @projects = $class->sort_objects( $metaproj->projects );

    if ( not @projects ) {
        STDOUT->print(
            qq{Unable to audit metaproj '$metaproj', },
            qq{since it has no projects yet.\n}
        );
        return 1;
    }

    STDOUT->print(
        qq{Performing $args{audits} audits for all },
        qq{projects in metaproj '$metaproj'\n}
    );

    $args{metaproj} = delete $args{name};

    my $fm_projects = EFS::ForkManager->new({

        parallel => 4,

        list => \@projects,

        action => sub {

            my $project = shift;

            my $object_id = $project->object_id(q{ });

            return EFS::RPC->execute(
                cell   => $project->cell,
                class  => q{EFS::CLI::Project},
                method => q{audit_rpc},
                args   => [
                    authorized => 1,
                    name       => $project->name,
                    %args,
                ],
                history => {
                    parentid => $args{history},
                    values   => {
                        request  => qq{audit project $object_id},
                        metaproj => $args{metaproj},
                        project  => $project->name,
                    },
                },
            );

        },

    });

    $fm_projects->run || die(
        qq{Unable to successfully audit the following projects:\n\t},
        join(qq{\n\t}, map { $_->object_id } $fm_projects->get_failed ), qq{\n}
    );

    STDOUT->print(
        qq{Successfully audited all projects in metaproj '$metaproj'\n}
    );

    return 1;

}

sub set_distgroup {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name      => { type => SCALAR },
        distgroup => { type => SCALAR },
    });

    my $feature_rs = $class->get_resultset( q{Feature} );

    if ( $feature_rs->configured( q{disable_restricted_access} ) ) {
        die(
            qq{Unable to set the distgroup attribute, since the\n},
            qq{disable_restricted_access feature has been enabled.\n},
        );
    }

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert(
        name => $args{name},
    );

    my $distgroup = EFS::Groupname->new({ name => $args{distgroup} });

    if ( $metaproj->distgroup eq $distgroup->name ) {
        STDOUT->print(
            qq{Metaproj $metaproj already has distgroup $distgroup\n},
        );
    } else {
        $metaproj->set_column( distgroup => $distgroup->name );
        $metaproj->update;
        STDOUT->print(
            qq{Set distgroup for metaproj $metaproj to $distgroup\n},
        );
    }

    return 1;

}

sub clear_distgroup {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR },
    });

    my $feature_rs = $class->get_resultset( q{Feature} );

    if ( $feature_rs->configured( q{disable_restricted_access} ) ) {
        die(
            qq{Unable to set the distgroup attribute, since the\n},
            qq{disable_restricted_access feature has been enabled.\n},
        );
    }

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert(
        name => $args{name},
    );

    if ( $metaproj->distgroup ) {
        $metaproj->set_column( distgroup => undef );
        $metaproj->update;
        STDOUT->print( qq{Cleared distgroup for metaproj $metaproj\n} );
    } else {
        STDOUT->print( qq{Metaproj $metaproj has no distgroup\n} );
    }

    return 1;

}

sub set_noworldaccess {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name          => { type => SCALAR },
        noworldaccess => { type => SCALAR },
    });

    my $feature_rs = $class->get_resultset( q{Feature} );

    if ( $feature_rs->configured( q{disable_restricted_access} ) ) {
        die(
            qq{Unable to set the noworldaccess attribute, since the\n},
            qq{disable_restricted_access feature has been enabled.\n},
        );
    }

    my $resultset = $class->get_resultset;

    my $metaproj = $resultset->retrieve_bynames_assert(
        name => $args{name},
    );

    my $noworldaccess = $args{noworldaccess} ? 1 : 0;

    $metaproj->set_column( noworldaccess => $noworldaccess );
    $metaproj->update;

    STDOUT->print(
        qq{Set noworldaccess for metaproj $metaproj to $noworldaccess\n},
    );

    return 1;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
