package EFS::CLI::Platform;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};
use Params::Validate qw( :all );

sub define {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name      => { type => SCALAR },
        backwards => { type => SCALAR | UNDEF, optional   => 1 },
        emulates  => { type => SCALAR | UNDEF, optional   => 1 },
    });

    $class->assert_platform_unused( $args{name} );

    my %create = ( name => $args{name} );

    my %objects = ();

    my $resultset = $class->get_resultset;

    foreach my $argument ( qw( backwards emulates ) ) {

        if ( $args{$argument} ) {

            if ( $args{name} eq $args{$argument} ) {
                die(
                    qq{Invalid $argument platform '$args{$argument}'\n},
                    qq{Matches platform name '$args{name}'\n},
                );
            }

            $objects{$argument} = $resultset->retrieve_bynames_assert(
                name => $args{$argument},
            );

            $create{$argument} = $objects{$argument}->id;

        }

    }

    my ($platform) = $resultset->search({ name => $args{name} });

    if ( $platform ) {

        STDOUT->print( qq{Platform $args{name} exists\n} );

        foreach my $key ( qw( backwards emulates ) ) {

            my $new = $objects{$key} ? $objects{$key}->id : q{};
            my $old = $platform->$key ? $platform->$key->id : q{};

            my $new_name = $objects{$key} ? $objects{$key}->name : q{NULL};

            if ( $old ne $new ) {
                $platform->set_column( $key => $new );
                $platform->update;
                STDOUT->print(
                    qq{Updated platform, set $key => $new_name\n}
                );
            }

        }

    } else {

        $platform = $resultset->create( \%create );
        STDOUT->print( qq{Defined platform $platform\n} );

    }

    return $platform;

}

sub undef {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type  => SCALAR },
    });

    my $resultset = $class->get_resultset;

    my $platform = $resultset->retrieve_bynames_assert( name => $args{name} );

    if ( my @installs = $platform->installs ) {
        die(
            qq{Unable to undef platform '$platform',\n},
            qq{as the following installs use this name:\n\t},
            join(qq{\n\t}, @installs ), qq{\n},
        );
    }

    if ( my @builds = $platform->builds ) {
        die(
            qq{Unable to undef platform '$platform',\n},
            qq{as the following builds use this name:\n\t},
            join(qq{\n\t}, @builds ), qq{\n},
        );
    }

    if ( my @aliases = $platform->aliases ) {
        die(
            qq{Unable to undef platform '$platform',\n},
            qq{as it has the following aliases:\n\t},
            join(qq{\n\t}, @aliases ), qq{\n},
        );
    }

    if ( my @instances = $platform->instances ) {
        die(
            qq{Unable to undef platform '$platform',\n},
            qq{as it has the following platform instances:\n\t},
            join(qq{\n\t}, @instances ), qq{\n},
        );
    }

    foreach my $type ( qw( forwards emulators ) ) {
        if ( my @depends = $platform->$type ) {
            die(
                qq{Unable to undef platform '$platform',\n},
                qq{as it has the following $type:\n\t},
                join( qq{\n\t}, @depends ), qq{\n},
            );
        }
    }

    my $object_id = $platform->object_id;
    $platform->delete;

    STDOUT->print( qq{Undefined platform $object_id\n} );

    return 1;

}

sub rename {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name    => { type => SCALAR },
        newname => { type => SCALAR },
    });

    $class->assert_maintenance_mode;

    my $name    = $args{name};
    my $newname = $args{newname};

    if ( $name eq $newname ) {
        die qq{Invalid arguments: new name must be different\n};
    }

    my $resultset = $class->get_resultset;
    my $platform  = $resultset->retrieve_bynames_assert( name => $name );

    $class->assert_platform_unused( $newname );

    if ( $resultset->search({ name => $newname })->count ) {
        die qq{Invalid argument: $newname is already a platform\n};
    }

    my $platform_alias_rs = $class->get_resultset( q{PlatformAlias} );

    $platform->set_column( name => $newname );
    $platform->update;

    $platform_alias_rs->create({
        name     => $name,
        platform => $platform->id,
    });

    STDOUT->print( qq{Renamed platform $name to $newname\n} );

    return 1;

}

sub getnames {

    my $class = shift;

    my $resultset = $class->get_resultset;

    my @platforms = $resultset->all;

    STDOUT->print( map { $_->name . qq{\n} } @platforms );

    return 1;

}

sub display {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    if ( not $args{name} ) {
        $class->display_objects( objects => [ $resultset->all ] );
        return 1;
    }

    my $platform = $resultset->retrieve_bynames_assert( name => $args{name} );

    $class->display_objects( objects => [ $platform ] );

    if ( my @aliases = $platform->aliases ) {
        $aliases[0]->display_objects( objects => \@aliases );
    }

    if ( my @instances = $platform->instances ) {
        $instances[0]->display_objects( objects => \@instances );
    }

    return 1;

}

sub history {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR },
        %EFS::CLI::History::validation_specs,
    });

    $args{platform} = delete $args{name};

    return EFS::CLI::History->history(%args);

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
