package EFS::DBI::Result::Install;

use strict;
use warnings;
use English;

use base q{EFS::DBI::Result};
use base q{EFS::DistableObject};

use Params::Validate qw( :all );
use File::Basename qw( dirname );
use Date::Format;
use Try::Tiny;

__PACKAGE__->load_components(qw/+EFS::DBI::Prefix Core/);

__PACKAGE__->table(q{install});

__PACKAGE__->add_columns(
    qw( release name spaceused filecount dircount
        linkcount checksum checktime cloneof
        devvolume distvolume )
);
__PACKAGE__->add_columns(
    id => { __PACKAGE__->get_id_column_attrs },
);
__PACKAGE__->set_primary_key(q{id});
__PACKAGE__->sequence(q{install_seq});

__PACKAGE__->belongs_to( devvolume  => q{EFS::DBI::Result::AFSVolume} );
__PACKAGE__->belongs_to( distvolume => q{EFS::DBI::Result::AFSVolume} );

__PACKAGE__->belongs_to( release => q{EFS::DBI::Result::Release} );
__PACKAGE__->belongs_to( cloneof => __PACKAGE__ );

__PACKAGE__->has_many( auditstates => q{EFS::DBI::Result::InstallAuditState} );
__PACKAGE__->has_many( cellstates  => q{EFS::DBI::Result::InstallCellState} );
__PACKAGE__->has_many( attrs       => q{EFS::DBI::Result::InstallAttrs} );

sub clones {
    my $self = shift;
    my $resultset = $self->get_resultset;
    my @clones = $resultset->search({ cloneof => $self->id });
    return @clones;
}

sub unclone {
    my $self = shift;
    $self->set_column( cloneof => undef );
    $self->update;
    return 1;
}

sub cell {
    return shift->release->cell;
}

sub metaproj {
    return shift->release->metaproj;
}

sub project {
    return shift->release->project;
}

# For DistableObject compliance
sub container {
    return shift->release;
}

sub stage {
    return shift->release->stage;
}

sub display_columns {
    return qw( metaproj project release
        name spaceused filecount
        dircount linkcount );
}

sub object_id {
    my $self = shift;
    my $separator = shift || q{/};
    return join( $separator,
                 $self->metaproj->name,
                 $self->project->name,
                 $self->release->name,
                 $self->name );
}

sub updatetime_for_cell {
    my $self = shift;
    my $cell = shift;
    my ($cellstate) = $self->cellstates({ cell => $cell->id }) or return;
    return $cellstate->updatetime;
}

sub get_diststate {
    return EFS::Diststate::Install->new({ install => shift });
}

sub get_filetree {
    return EFS::FileTree->new({ pathname => shift->get_path(q{dev}) });
}

sub get_path {

    my $self     = shift;
    my $pathtype = shift || q{dist};

    my $name = $self->name;

    my $dir = $self->release->get_path($pathtype);
    $dir .= q{/install} if $pathtype eq q{dev};
    $dir .= q{/.exec} if $name ne q{common};
    $dir .= qq{/$name};

    return $dir;

}

sub assert_not_disabled {
    my $self = shift;
    if ( my $disabled = $self->is_disabled ) {
        die qq{Install '$self' is disabled: $disabled\n};
    }
    return;
}

sub display_translate {

    my $class = shift;

    my ($key,$value) = @_;

    if ( $key eq q{spaceused} ) {
        return $class->size_shorthand($value);
    }
    else {
        return $value;
    }

}

sub not_empty {
    my $self = shift;
    my $dir = $self->get_path(q{dev});
    my $io = IO::Dir->new($dir) ||
        die qq{Unable to opendir $dir: $ERRNO\n};
    return grep { not $_ ~~ [qw(. ..)] } $io->read;
}

sub is_empty {
    return not shift->not_empty;
}

sub validate_audits {

    my $self = shift;

    my $diststate = $self->get_diststate;
    my $checktime = $diststate->get_checktime;

    my @audits = EFS::InstallAudit->get_required();

    my %failed = ();

    if ( $self->is_empty and not $checktime ) {
        ##  no critic (MagicNumbers)
        return -1;
    }

    if ( not $checktime ) {
        STDOUT->print( qq{Install '$self' has no checktime.\n} );
        return 0;
    }

    foreach my $audit (@audits) {

        my ($auditstate) = $self->auditstates( { name => $audit } );

        if ($auditstate) {
            if ( not $auditstate->status ) {
                STDOUT->print( qq{Install '$self' has failed the $audit audit.\n} );
                $failed{$audit}++;
            }
        }
        else {
            STDOUT->print( qq{Install '$self' has not run the $audit audit.\n} );
            $failed{$audit}++;
        }

        if ( $failed{$audit} ) {
            next;
        }

        my $audittime = $auditstate->audittime;

        if ( $audittime < $checktime ) {
            STDOUT->print(
                qq{Install '$self' has been modified since the $audit },
                qq{audit was last run.\n},
                q{Checktime => },
                time2str( q{[%Y-%m-%d %H:%M:%S]}, $checktime, q{GMT} ), qq{\n},
                q{Audittime => },
                time2str( q{[%Y-%m-%d %H:%M:%S]}, $audittime, q{GMT} ), qq{\n},
            );
            $failed{$audit}++;
            next;
        }

    }

    if ( keys %failed ) {
        return 0;
    }
    else {
        return 1;
    }

}

sub validate_checkpoint {

    my $self = shift;

    my (%args) = validate( @_, {
        filetree  => { isa => q{EFS::FileTree}, optional => 1 },
        diststate => { isa => q{EFS::Diststate::Install}, optional => 1 },
    });

    my $diststate = $args{diststate} || $self->get_diststate;

    my $checktime = $diststate->get_checktime;

    my $filetree = $args{filetree} || $self->get_filetree;

    if ( $self->is_empty and not $checktime ) {
        ##  no critic (MagicNumbers)
        return -1;
    }

    if ( not $checktime ) {
        STDOUT->print( qq{Install '$self' has no checktime.\n} );
        return 0;
    }

    my $max_ctime = $filetree->get_max_ctime( show => $checktime ) ||
        die qq{Unable to calculate change time of install '$self'\n};

    if ( $max_ctime > $checktime ) {
        STDOUT->print( qq{Install '$self' has been modified since the last checktime.\n} );
        return 0;
    }

    return 1;

}

sub update_checkpoint {

    my $self = shift;

    my (%args) = validate( @_, {
        history  => { type => SCALAR },
        nolock   => { type => BOOLEAN, optional => 1 },
        noaudit  => { type => BOOLEAN, optional => 1 },
        filetree => { isa  => q{EFS::FileTree}, optional => 1 },
    });

    my $is_empty = $self->is_empty;

    my $diststate = $self->get_diststate;

    my $checktime = $diststate->get_checktime;

    if (   $is_empty
        && ( not $diststate->cells_partially_updated )
        && $checktime ) {
        # IOW, if its empty, never been disted, and has a checktime
        # timestamp...
        $self->set_column( checktime => 0 );
        $self->update;
        STDOUT->print(
            q{Removing bogus checktime for empty, },
            qq{unreleased install '$self'.\n}
        );
        return 1;
    }

    if ( $is_empty && ( not $checktime ) ) {
        STDOUT->print( qq{Checktime not updated for empty install '$self'.\n} );
        return 1;
    }

    if ( not $args{nolock} ) {
        EFS::CLI::Install->lock(
            history         => $args{history},
            metaproj        => $self->metaproj->name,
            project         => $self->project->name,
            release         => $self->release->name,
            name            => $self->name,
            _release_locked => 1,
        );
    }

    my $filetree = $args{filetree} || $self->get_filetree;

    try {

        if ( $args{noaudit} ) {
            STDOUT->print(
                qq{Skipping audits for install '$self' (-noaudit).\n} );
        }
        else {
            EFS::CLI::Install->audit(
                history         => $args{history},
                metaproj        => $self->metaproj->name,
                project         => $self->project->name,
                release         => $self->release->name,
                name            => $self->name,
                filetree        => $filetree,
                _release_locked => 1,
            );
        }


        if ($checktime) {

            my $max_ctime = $filetree->get_max_ctime( show => $checktime ) || die(
                q{Unable to calculate change time },
                qq{of install '$self'\n},
            );

            if ( $checktime < $max_ctime ) {
                $self->set_column( checktime => $max_ctime );
                $self->update;
                STDOUT->print( qq{Updated checktime for modified install '$self'.\n} );
            }
            else {
                STDOUT->print( qq{Unmodified install '$self' skipped.\n} );
            }

        }
        else {

            my $max_ctime = $filetree->get_max_ctime || die(
                q{Unable to calculate change time },
                qq{of install '$self'\n},
            );
            $self->set_column( checktime => $max_ctime );
            $self->update;
            STDOUT->print( qq{Updated initial checktime for install '$self'\n} );

        }

    }
    catch {

        if ( not $args{nolock} ) {

            EFS::CLI::Install->unlock(
                history         => $args{history},
                metaproj        => $self->metaproj->name,
                project         => $self->project->name,
                release         => $self->release->name,
                name            => $self->name,
                _release_locked => 1,
            );
        }

        die qq{Unable to checktime install '$self': $_};

    };

    return 1;

}

sub distribute {

    my $self = shift;

    my (%args) = validate( @_, {
        diststats_id => { type => SCALAR },
        src_cell     => { isa  => q{EFS::DBI::Result::Cell} },
        dest_cell    => { isa  => q{EFS::DBI::Result::Cell} },
        verbose      => { type => BOOLEAN, optional => 1 },
        query        => { type => BOOLEAN, optional => 1 },
    });

    my $src_cell  = $args{src_cell};
    my $dest_cell = $args{dest_cell};

    if ( $dest_cell->fstype eq q{afs} ) {
        die qq{Unable to distributed to AFS cells yet\n};
        # XXX: This is a LOT of work....
    }

    my $src_path    = $self->get_source_path(%args);

    my $release_mpr = $self->release->object_id;
    my $name        = $self->name;
    my $dest_admin  = $dest_cell->admin_path( type => q{dist} );
    my $dest_path   = $self->get_destination_path(%args);

    if ( my $clone = $self->cloneof ) {

        my $clone_path = $clone->get_destination_path(%args);

        my $pathinfo = EFS::RPC->execute(
            cell   => $dest_cell,
            class  => q{EFS::Util},
            method => q{GetPathInfo},
            args   => [ $clone_path ],
        );

        if ( $pathinfo->{type} eq q{dir} ) {
            EFS::RPC->execute(
                cell   => $dest_cell,
                class  => q{EFS::Rsync},
                method => q{new},
                args   => [{
                    src_path  => $clone_path,
                    dest_path => $dest_path,
                }],
            );
        }

    }

    my $rsync = EFS::Rsync->new({
        diststats_id => $args{diststats_id},
        install      => $self,
        src_cell     => $src_cell,
        dest_cell    => $dest_cell,
        src_path     => $src_path,
        dest_path    => $dest_path,
        query        => $args{query},
        verbose      => $args{verbose},
    });

    $rsync->display_stats;

    STDOUT->print( qq{Successfully distributed $self to $args{dest_cell}\n} );

    return 1;

}

sub get_source_path {

    my $self = shift;
    my %args = @_;

    my $src_cell  = $args{src_cell};
    my $dest_cell = $args{dest_cell};

    my $release_mpr = $self->release->object_id;
    my $name        = $self->name;

    my $src_type    = $dest_cell->celltype ne q{dev} ? q{dist} : q{dev};
    my $src_admin   = $src_cell->admin_path( type => $src_type );

    my $dir = qq{$src_admin/$src_type/$release_mpr};
    $dir .= q{/install} if $src_type eq q{dev};
    $dir .= q{/.exec} if $name ne q{common};
    $dir .= qq{/$name};

    return $dir;

}

sub get_destination_path {

    my $self = shift;
    my %args = @_;

    my $src_cell  = $args{src_cell};
    my $dest_cell = $args{dest_cell};

    my $release_mpr = $self->release->object_id;
    my $name        = $self->name;

    my $dest_admin  = $dest_cell->admin_path( type => q{dist} );

    my $dir = qq{$dest_admin/dist/$release_mpr};
    $dir    .= q{/.exec} if $name ne q{common};
    $dir    .= qq{/$name};

    return $dir;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
