package EFS::DBI::Result::Project;

use strict;
use warnings;
use English;

use feature q{switch};

use base q{EFS::DBI::Result};
use base q{EFS::DistableObject};

use Date::Format;

__PACKAGE__->load_components( qw/+EFS::DBI::Prefix Core/ );

__PACKAGE__->table( q{project} );

__PACKAGE__->add_columns(
    id            => { __PACKAGE__->get_id_column_attrs },
    metaproj      => { efs_column_group => [qw( display )] },
    name          => { efs_column_group => [qw( display )] },
    cell          => { efs_column_group => [qw( display )] },
    movelock      => { efs_column_group => [qw( display )] },
    owner_list    => { efs_column_group => [qw( optional )] },
    notify_list   => { efs_column_group => [qw( optional )] },
    devvolume     => { efs_column_group => [qw( optional )] },
    distvolume    => { efs_column_group => [qw( optional )] },
    noworldaccess => { efs_column_group => [qw( access )] },
    distgroup     => { efs_column_group => [qw( access )] },
);

__PACKAGE__->set_primary_key( q{id} );
__PACKAGE__->sequence( q{project_seq} );

__PACKAGE__->belongs_to( metaproj => q{EFS::DBI::Result::Metaproj} );
__PACKAGE__->belongs_to( cell     => q{EFS::DBI::Result::Cell} );

__PACKAGE__->belongs_to( devvolume  => q{EFS::DBI::Result::AFSVolume} );
__PACKAGE__->belongs_to( distvolume => q{EFS::DBI::Result::AFSVolume} );

__PACKAGE__->has_many( releases       => q{EFS::DBI::Result::Release} );
__PACKAGE__->has_many( releasealiases => q{EFS::DBI::Result::ReleaseAlias} );

__PACKAGE__->has_many( attrs => q{EFS::DBI::Result::ProjectAttrs} );

sub display_columns {
    return qw(metaproj name cell spaceused
              filecount dircount linkcount);
}

sub subitems {
    my $self = shift;
    return ( $self->releases, $self->releasealiases );
}

sub container {
    return shift->metaproj;
}

sub active_releases {
    return grep { not $_->destroyed } shift->releases;
}

sub destroyed_releases {
    return grep { $_->destroyed } shift->releases;
}

sub object_id {
    my $self = shift;
    my $separator = shift || q{/};
    return join( $separator,
                 $self->metaproj->name,
                 $self->name );
}

sub installs {
    return map { $_->installs(@_) } shift->releases;
}

sub get_path {
    my ($self,$pathtype) = @_;
    $pathtype ||= q{dist};
    return $self->metaproj->get_path($pathtype) . q{/} . $self->name;
}

sub assert_not_disabled {

    my $self = shift;

    if ( my $disabled = $self->is_disabled ) {
        die qq{Project '$self' is disabled: $disabled\n};
    }

    return;
}

sub autopath {

    my $self = shift;

    my $cell = $self->cell;

    my $autopath = q{};

    given ( $cell->fstype ) {

        when ( q{afs} ) {
            $autopath = join(
                q{/},
                q{/afs},
                $cell->name,
                q{efs/dev},
                $self->object_id,
            );
        }

        when ( q{nfs} ) {
            my ($fileshare) = $cell->fileshares({ filetype => q{dev} });
            my ($fileqtree) = $fileshare->fileqtrees({ name => q{efs_qtree} });
            $autopath = join(
                q{/},
                q{/.efsglobal},
                $fileshare->autoname,
                $fileqtree->name,
                q{dev},
                $self->object_id,
            );
        }

    }

    return $autopath;

}

# XXX: Not sure about this one...
sub move_backout {

    my $self = shift;
    my %args = @_;

    $self->set_column( movelock => undef );
    $self->update;

    EFS::CLI::Namespace->addentry(
        type     => q{dev},
        realpath => $self->object_id,
        autopath => $self->autopath,
    );

    return die(
        qq{Unable to complete move of project to $args{tocell}:\n},
        $args{message},
    );

}

sub get_diststate {
    return EFS::Diststate::Project->new({ project => shift });
}

sub assert_auto_project {

    my $self = shift;

    my $release_qr  = qr{ ^ \d{8} - \d{6} $ }msx;
    my @alias_names = qw( current next );

    my @releases = grep { not $_->name =~ $release_qr } $self->releases;
    my @aliases  = grep { not $_->name ~~ \@alias_names } $self->releasealiases;

    if ( not @releases and not @aliases ) {
        return 1;
    }

    my @message = ();

    if ( @releases ) {
        push @message, (
            qq{The following releases are not autorelease compliant:\n\t},
            join( qq{\n\t}, @releases ), qq{\n},
        );
    }

    if ( @aliases ) {
        push @message, (
            qq{The following release aliases are not autorelease compliant:\n\t},
            join( qq{\n\t}, @aliases ), qq{\n},
        );
    }

    return die( @message );

}

sub display_translate {

    my $class = shift;
    my ($key,$value) = @_;

    if ( $key eq q{spaceused} ) {
        return $class->size_shorthand($value);
    }
    else {
        return $value;
    }

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
