#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 49;

use lib q{test/lib};
use EFS::Test;

use blib;
use EFS;

my $test = EFS::Test->new;
my $config = $test->get_config;

my $username  = $test->get_primary_username;
my $groupname = $test->get_primary_groupname;

my $root_domain     = $config->get_value( q{Global}, q{root_domain} );
my $secondary_user  = $config->get_value( q{Global}, q{secondary_user} );
my $secondary_group = $config->get_value( q{Global}, q{secondary_group} );

my $ownerlist = join q{@}, $username, $root_domain;

my @setup = (
    q{efs display metaproj testmeta},
    q{efs display project testmeta testproj},
    q{efs create release testmeta testproj testrel},
    q{efs create install testmeta testproj testrel common},
);

foreach my $setup (@setup) {
    $test->efs_test_command(
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

check_dirs(
    common_user  => $username,
    common_group => $groupname,
    src_user     => $username,
    src_group    => $groupname,
    build_user   => $username,
    build_group  => $groupname,
);

{
    my @expected = (
        qq{Updated metaproj 'testmeta', set username to '$secondary_user'}
    );
    push @expected =>
        qq{Updated metaproj 'testmeta', set groupname to '$secondary_group'}
        if $groupname ne $secondary_group;

    $test->efs_test_command(
        run_as  => q{efsops},
        name    => q{change metaproj ownership},
        command => qq{efs create metaproj testmeta $secondary_user $secondary_group $ownerlist},
        filter  => qr{Updated \s+ metaproj}msx,
        expect  => \@expected,
    );
}

$test->efs_test_command(
    run_as  => q{efsops},
    name    => q{chown release},
    command => q{efs chown release testmeta testproj testrel},
    filter  => qr{chowning|successfully}imsx,
    expect  => [
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/src},
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/build},
        q{Chowning all installs in release 'testmeta/testproj/testrel'},
        q{Successfully chowned all installs in release 'testmeta/testproj/testrel'},
    ],
);

check_dirs(
    common_user  => $secondary_user,
    common_group => $secondary_group,
    src_user     => $secondary_user,
    src_group    => $secondary_group,
    build_user   => $secondary_user,
    build_group  => $secondary_group,
);

$test->efs_test_command(
    run_as  => q{efsops},
    name    => q{chown with recurse},
    command => q{efs chown release testmeta testproj testrel},
    filter  => qr{chowning|successfully}imsx,
    expect  => [
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/src },
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/build },
        q{Chowning all installs in release 'testmeta/testproj/testrel' },
        q{Successfully chowned all installs in release 'testmeta/testproj/testrel' },
    ],
);

check_dirs(
    common_user  => $secondary_user,
    common_group => $secondary_group,
    src_user     => $secondary_user,
    src_group    => $secondary_group,
    build_user   => $secondary_user,
    build_group  => $secondary_group,
);

$test->efs_test_command(
    run_as  => q{efsops},
    name    => q{lock release},
    command => q{efs lock release testmeta testproj testrel},
    filter  => qr{locking|successfully}imsx,
    expect  => [
        q{Locking directory /efs/dev/testmeta/testproj/testrel/src},
        q{Locking directory /efs/dev/testmeta/testproj/testrel/build},
        q{Locking all installs in release 'testmeta/testproj/testrel'},
        q{Locking install /efs/dev/testmeta/testproj/testrel/install/common},
        q{Successfully chowned all installs in testmeta/testproj/testrel},
    ],
);

check_dirs(
    common_user  => q{root},
    common_group => q{root},
    src_user     => q{root},
    src_group    => q{root},
    build_user   => q{root},
    build_group  => q{root},
);

$test->efs_test_command(
    run_as  => q{efsops},
    name    => q{chown with no recurse (default)},
    command => q{efs chown release testmeta testproj testrel},
    filter  => qr{chowning|successfully}imsx,
    expect  => [
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/src},
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/build},
        q{Chowning all installs in release 'testmeta/testproj/testrel'},
        q{Successfully chowned all installs in release 'testmeta/testproj/testrel'},
    ],
);

check_dirs(
    common_user  => q{root},
    common_group => q{root},
    src_user     => q{root},
    src_group    => q{root},
    build_user   => q{root},
    build_group  => q{root},
);

$test->efs_test_command(
    run_as  => q{efsops},
    name    => qq{unlock and set ownership back to $secondary_user/$secondary_group},
    command => q{efs unlock release testmeta testproj testrel},
    filter  => qr{unlocking|successfully}imsx,
    expect  => [
        q{Unlocking directory /efs/dev/testmeta/testproj/testrel/src},
        q{Unlocking directory /efs/dev/testmeta/testproj/testrel/build},
        q{Unlocking all installs in release 'testmeta/testproj/testrel'},
        q{Unlocking install /efs/dev/testmeta/testproj/testrel/install/common},
        q{Successfully chowned all installs in testmeta/testproj/testrel},
    ],
);

check_dirs(
    common_user  => $secondary_user,
    common_group => $secondary_group,
    src_user     => $secondary_user,
    src_group    => $secondary_group,
    build_user   => $secondary_user,
    build_group  => $secondary_group,
);

{
    my @expected = (
        qq{Updated metaproj 'testmeta', set username to '$username'}
    );
    push @expected => qq{Updated metaproj 'testmeta', set groupname to '$groupname'}
        if $groupname ne $secondary_group;

    $test->efs_test_command(
        run_as  => q{efsops},
        name    => q{revert metaproj to original setting},
        command =>
            qq{efs create metaproj testmeta $username $groupname $ownerlist},
        filter => qr{updated}imsx,
        expect => \@expected,
    );
}

# Everything else from here on out runs as the primary user
$test->become_user( q{primary_user} );

$test->efs_test_command(
    name    => q{revert project and release to original setting},
    command => q{efs chown release testmeta testproj testrel},
    filter  => qr{chowning|successfully}imsx,
    expect  => [
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/src},
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/build},
        q{Chowning all installs in release 'testmeta/testproj/testrel'},
        q{Successfully chowned all installs in release 'testmeta/testproj/testrel'},
    ],
);

check_dirs(
    common_user  => $username,
    common_group => $groupname,
    src_user     => $username,
    src_group    => $groupname,
    build_user   => $username,
    build_group  => $groupname,
);

# NOTE: Should we abort if there's no other valid users, or just skip
# the tests?

$test->efs_test_command(
    name    => q{valid alternate group member via -username},
    command => qq{efs chown release testmeta testproj testrel -username $secondary_user},
    filter  => qr{chowning|successfully}imsx,
    expect  => [
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/src},
        q{Chowning directory /efs/dev/testmeta/testproj/testrel/build},
        q{Chowning all installs in release 'testmeta/testproj/testrel'},
        q{Successfully chowned all installs in release 'testmeta/testproj/testrel'},
    ],
);

check_dirs(
    common_user  => $secondary_user,
    common_group => $groupname,
    src_user     => $secondary_user,
    src_group    => $groupname,
    build_user   => $secondary_user,
    build_group  => $groupname,
);

$test->efs_test_command(
    name    => q{lock release again},
    command => q{efs lock release testmeta testproj testrel},
    filter  => qr{locking|successfully}imsx,
    expect  => [
        q{Locking directory /efs/dev/testmeta/testproj/testrel/src},
        q{Locking directory /efs/dev/testmeta/testproj/testrel/build},
        q{Locking all installs in release 'testmeta/testproj/testrel'},
        q{Locking install /efs/dev/testmeta/testproj/testrel/install/common},
        q{Successfully chowned all installs in testmeta/testproj/testrel},
    ],
);

check_dirs(
    common_user  => q{root},
    common_group => q{root},
    src_user     => q{root},
    src_group    => q{root},
    build_user   => q{root},
    build_group  => q{root},
);

$test->efs_test_command(
    name    => q{verify unlock works with -username},
    command => qq{efs unlock release testmeta testproj testrel -username $secondary_user},
    filter  => qr{locking|successfully}imsx,
    expect  => [
        q{Unlocking directory /efs/dev/testmeta/testproj/testrel/src},
        q{Unlocking directory /efs/dev/testmeta/testproj/testrel/build},
        q{Unlocking all installs in release 'testmeta/testproj/testrel'},
        q{Unlocking install /efs/dev/testmeta/testproj/testrel/install/common},
        q{Successfully chowned all installs in testmeta/testproj/testrel},
    ],
);

check_dirs(
    common_user  => $secondary_user,
    common_group => $groupname,
    src_user     => $secondary_user,
    src_group    => $groupname,
    build_user   => $secondary_user,
    build_group  => $groupname,
);

$test->efs_test_command(
    name    => q{invalid alternate group member via -username},
    command => q{efs chown release testmeta testproj testrel -username nobody},
    not     => 1,
    expect  => [
        q{Invalid username 'nobody'.},
        qq{Must be a member of the group '$groupname'},
    ],
);

my @teardown = (
    q{efs setstage release testmeta testproj testrel rejected},
    q{efs destroy release testmeta testproj testrel},
    q{efs purge release testmeta testproj testrel -now},
);

foreach my $teardown (@teardown) {
    $test->efs_test_command(
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;

sub check_dirs {

    my %args = @_;

    $test->efs_test_directory(
        name      => q{check owner/group of install/common directory},
        directory => q{/efs/dev/testmeta/testproj/testrel/install/common},
        user      => $args{common_user},
        group     => $args{common_group},
    );

    $test->efs_test_directory(
        name      => q{check owner/group of src directory},
        directory => q{/efs/dev/testmeta/testproj/testrel/src},
        user      => $args{src_user},
        group     => $args{src_group},
    );

    $test->efs_test_directory(
        name      => q{check owner/group of build directory},
        directory => q{/efs/dev/testmeta/testproj/testrel/build},
        user      => $args{build_user},
        group     => $args{build_group},
    );

    return 1;

}
