#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 39;

use lib q{test/lib};
use EFS::Test;

my $test = EFS::Test->new;

my $install = q{x86-64.rhel.5};

my $mpr       = q{testmeta testproj testrel};
my $mpr_path  = q{testmeta/testproj/testrel};
my $mpd       = q{testmeta testproj testdep};
my $mpd_path  = q{testmeta/testproj/testdep};
my $mpri_path = qq{$mpr_path/$install};

my $install_path = qq{/efs/dev/$mpr_path/install/.exec/$install};

my @setup = (

    q{efs display metaproj testmeta},
    q{efs display project testmeta testproj},
    q{efs create release testmeta testproj testrel},
    q{efs create release testmeta testproj testdep},

    qq{efs create install testmeta testproj testrel $install},

    qq{mkdir /efs/dev/$mpr_path/install/.exec/$install/lib},
    qq{touch /efs/dev/$mpr_path/install/.exec/$install/lib/no_mpr.testrpath},
    qq{touch /efs/dev/$mpr_path/install/.exec/$install/lib/no_dev.testrpath},
    qq{touch /efs/dev/$mpr_path/install/.exec/$install/lib/no_exec.testrpath},
    qq{touch /efs/dev/$mpr_path/install/.exec/$install/lib/pass.testrpath},

    qq{mkdir /efs/dev/$mpr_path/install/.exec/$install/bin},
    qq{touch /efs/dev/$mpr_path/install/.exec/$install/bin/core},
    qq{touch /efs/dev/$mpr_path/install/.exec/$install/bin/emacs.backup~},
    qq{touch /efs/dev/$mpr_path/install/.exec/$install/bin/rcs.control,v},

    qq{touch /efs/dev/$mpr_path/install/.exec/$install/bin/__dummy__},

);

foreach my $setup (@setup) {
    $test->efs_test_command(
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

foreach my $ffrule (qw( cores rcs emacs )) {
    $test->efs_test_command(
        name    => qq{(setup) check for $ffrule ffrule},
        command => qq{efs display ffrule $ffrule},
        expect  => [
            qr{Ffrule \s+ Regex \s+ Description}xms,
            qr{=+}msx,
            qr{^$ffrule\s+}msx,
        ],
        bail    => 1,
    );
}

my $fakereadelf = $ENV{PWD} . q{/test/bin/fakereadelf};

$test->efs_test_command(
    name    => q{efs audit install, dependencies},
    command => qq{efs audit install testmeta testproj testrel $install -audits dependencies -fakereadelf $fakereadelf},
    not     => 1,
    expect  => [
        qq{Performing dependencies audit on install $mpri_path},
        qq{Defined dependency of $mpr_path on $mpd_path, type runtime},
        q{The following audits produced errors:},
        q{Audit 'dependencies':},
        qq{Filename: $install_path/lib/no_dev.testrpath},
        qq{RPATH items with '/efs/dev/..' are not allowed: /efs/dev/$mpr_path/install/.exec/$install/lib},
        qq{Filename: $install_path/lib/no_exec.testrpath},
        qq{RPATH items with 'exec' (rather than '.exec') are not allowed: /efs/dist/$mpr_path/exec/lib},
        qq{Filename: $install_path/lib/no_mpr.testrpath},
        qq{RPATH item does not exist: /efs/dist/nometaproj/noproject/norelease/.exec/$install/lib},
        q{Unable to successfully complete or pass all audits},
    ],
);

$test->efs_test_command(
    name    => q{efs audit install, forbidden files},
    command => qq{efs audit install testmeta testproj testrel $install }
        . q{-audits forbidden_files},
    not     => 1,
    expect  => [
        qq{Performing forbidden_files audit on install $mpri_path},
        q{The following audits produced errors:},
        q{Audit 'forbidden_files':},
        qq{Forbidden file rule 'cores' violated by $install_path/bin/core},
        qq{Forbidden file rule 'emacs' violated by $install_path/bin/emacs.backup~},
        qq{Forbidden file rule 'rcs' violated by $install_path/bin/rcs.control,v},
        q{Unable to successfully complete or pass all audits},
    ],
);

$test->efs_test_command(
    name    => q{efs display install},
    command => qq{efs display install testmeta testproj testrel $install},
    expect  => [
        qr{Metaproj \s+ Project \s+ Release \s+ Install}xms,
        qr{=+}msx,
        qr{testmeta \s+ testproj \s+ testrel \s+ $install}xms,
        qr{Metaproj \s+ Project \s+ Release \s+ Install}xms,
        qr{=+}msx,
        qr{testmeta \s+ testproj \s+ testrel \s+ $install \s+ dependencies \s+ \S+ \s+ \S+ \s+ \(GMT\) \s+ 0}xms,
        qr{testmeta \s+ testproj \s+ testrel \s+ $install \s+ forbidden_files \s+ \S+ \s+ \S+ \s+ \(GMT\) \s+ 0}xms,
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    name    => q{define dummy ffrule},
    command => q{efs define ffrule dummy __dummy__ dummy},
    expect  => [ q{Defined forbidden file rule dummy}, ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    name    => q{define dummy ffrule, again},
    command => q{efs define ffrule dummy __dummy__ dummy},
    expect  => [ q{Forbidden file rule dummy is already defined}, ],
);

$test->efs_test_command(
    name    => q{efs audit install, forbidden files, with dummy rule},
    command => qq{efs audit install testmeta testproj testrel $install -audits forbidden_files},
    not     => 1,
    filter  => qr{Forbidden \s+ file \s+ rule \s+ .dummy.}msx,
    expect  => qq{Forbidden file rule 'dummy' violated by $install_path/bin/__dummy__},
);

$test->efs_test_command(
    run_as  => q{efsops},
    name    => q{undef dummy ffrule},
    command => q{efs undef ffrule dummy},
    expect  => q{Undefined forbidden file rule dummy},
);

$test->efs_test_command(
    run_as  => q{efsops},
    name    => q{undef dummy ffrule, again},
    command => q{efs undef ffrule dummy},
    not     => 1,
    expect  => q{No such forbidden file rule dummy},
);

# Everything from here on out runs as the primary user
$test->become_user( q{primary_user} );

$test->efs_test_command(
    command => qq{rm -rf /efs/dev/$mpr_path/install/.exec/$install/bin},
);

$test->efs_test_command(
    command => qq{rm -rf /efs/dev/$mpr_path/install/.exec/$install/lib/*.testrpath},
);

$test->efs_test_command(
    command => qq{touch /efs/dev/$mpr_path/install/.exec/$install/lib/pass.testrpath},
);

$test->efs_test_command(
    command => qq{efs audit install testmeta testproj testrel $install -fakereadelf $fakereadelf},
    expect  => [
        qq{Performing dependencies audit on install $mpri_path},
        qq{Performing forbidden_files audit on install $mpri_path},
    ],
);

$test->efs_test_command(
    command => qq{efs audit install testmeta testproj testrel $install -fakereadelf $fakereadelf},
    expect  => [
        qr{Skipping\s+dependencies\s+audit\s+for\s+'$mpri_path',\s+passed\s+on\s+\S+\s+\S+\s+\(GMT\)}msx,
        qr{Skipping\s+forbidden_files\s+audit\s+for\s+'$mpri_path',\s+passed\s+on\s+\S+\s+\S+\s+\(GMT\)}msx,
    ],
);

$test->efs_test_command(
    command => qq{efs display install testmeta testproj testrel $install},
    expect  => [
        qr{Metaproj \s+ Project \s+ Release \s+ Install}xms,
        qr{=+}msx,
        qr{testmeta \s+ testproj \s+ testrel \s+ $install}xms,
        qr{Metaproj \s+ Project \s+ Release \s+ Install}xms,
        qr{=+}msx,
        qr{testmeta \s+ testproj \s+ testrel \s+ $install \s+ dependencies \s+ \S+ \s+ \S+ \s+ \(GMT\) \s+ 1}xms,
        qr{testmeta \s+ testproj \s+ testrel \s+ $install \s+ forbidden_files \s+ \S+ \s+ \S+ \s+ \(GMT\) \s+ 1}xms,
    ],
);

$test->efs_test_command(
    command => q{efs history ffrule dummy -count 4},
    expect  => [
        qr{History \s+ ID \s+ StartTime \s+ Duration \s+ Status \s+ Authuser \s+ Request}xms,
        qr{-+}msx,
        qr{define \s+ ffrule \s+ dummy}xms,
        qr{define \s+ ffrule \s+ dummy}xms,
        qr{undef \s+ ffrule \s+ dummy}xms,
        qr{undef \s+ ffrule \s+ dummy}xms,
    ],
);

my @teardown = (
    q{efs setstage release testmeta testproj testrel rejected},
    q{efs destroy release testmeta testproj testrel},
    q{efs purge release testmeta testproj testrel -now},
    q{efs setstage release testmeta testproj testdep rejected},
    q{efs destroy release testmeta testproj testdep},
    q{efs purge release testmeta testproj testdep -now},
);

foreach my $teardown (@teardown) {
    $test->efs_test_command(
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;

