#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 40;

use lib q{test/lib};
use EFS::Test;

my $test = EFS::Test->new;
my $config = $test->get_config;

my $ldc_name      = $config->get_value( q{LocalDevCell}, q{name} );
my $ldc_shortname = $config->get_value( q{LocalDevCell}, q{shortname} );

my $rdc_name      = $config->get_value( q{RemoteDevCell}, q{name} );
my $rdc_shortname = $config->get_value( q{RemoteDevCell}, q{shortname} );

my $mp_name = q{testmeta testnsra};
my $mp_path = q{testmeta/testnsra};

my @setup = (
    q{efs display metaproj testmeta},
    qq{efs create project $mp_name $ldc_name},
);

foreach my $setup (@setup) {
    $test->efs_test_command(
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

my @releases = qw( 1.0.0 1.0.1 dependent );

my @releasealiases = ( [qw(1.0 1.0.1)], );

my @depends = ( [qw( dependent 1.0 )], );

# Setup everything...
create_releases(@releases);
update_releases(@releases);
checknolock_releases(@releases);
dist_releases(qw( 1.0.0 1.0.1 ));

# Now the real tests start

# First, let's verify that disttree ONLY updates releasealiases in
# missing cells

$test->efs_test_command(
    command => qq{efs create releasealias $mp_name 1.0 1.0.0},
);

setup_dependencies( \@depends, q{define} );

$test->efs_test_command(
    command => qq{efs dist releasealias $mp_name 1.0 -cells $ldc_shortname},
);

$test->efs_test_command(
    command => qq{efs create releasealias $mp_name 1.0 1.0.1},
);

# Now we have 1.0 -> 1.0.1 in /efs/dev, and 1.0 -> 1.0.0 in LDC's
# /efs/dist, and NOTHING in RDC's /efs/dist.  Only RDC should get the
# new releasealias.

$test->efs_test_command(
    command => qq{efs disttree release $mp_name dependent --dryrun --global},
    expect  => [
        qr{Stage \s+ Depth \s+ Diststate \s+Runtime \s+ Dependencies}xms,
        qr{=+}msx,
        qr{dev \s+ 1 \s+ 2d \s+ $mp_path/dependent}xms,
        qr{dev \s+ 2 \s+ 1d \s+ $mp_path/1.0}xms,
        qr{dev \s+ 3 \s+ $mp_path/1.0.1}xms,
    ],
);

$test->efs_test_command(
    command   => qq{efs disttree release $mp_name dependent --global},
    filternot => qr{ ^ (Created|Changed) \s+ (directory|ownership) }msx,
    expect    => [
        q{Skipping 1 MPR at depth 3, which is up to date},
        q{Nothing left to distribute at depth 3},
        q{Distributing 1 MPR at depth 2:},
        qq{$mp_path/1.0},
        dist_releasealias_output( q{1.0}, q{1.0.1} ),
        q{Distributing 1 MPR at depth 1:},
        qq{$mp_path/dependent},
        dist_release_output(q{dependent}),
        q{Skipped the following up to date MPRs:},
        qq{$mp_path/1.0.1},
        q{Successfully disted the following MPRs:},
        qq{$mp_path/1.0},
        qq{$mp_path/dependent},
    ],
);

# Then, verify that dist release with and w/out -nsra does what we
# expect.

update_releases(q{dependent});
checknolock_releases(q{dependent});

$test->efs_test_command(
    command => qq{efs dist release $mp_name dependent --global},
    not     => 1,
    expect  => [
        qq{Diststate for releasealias dependency $mp_path/1.0:},
        q{Releasealias target: 1.0.1},
        q{Dev environment diststate:},
        qr{Cell \s+ Target}xms,
        qr{=+}msx,
        qr{$ldc_shortname \s+ 1.0.0}xms,
        qr{-+}msx,
        q{1 cell is up to date and not displayed.},
        q{Legend:},
        q{NNN: Releasealias is disted but target is not up to date},
        q{---: Releasealias is missing from cell},
        q{Unable to dist release because the following releasealias},
        q{dependencies are not consistently deployed, and have different},
        q{values in different cells.},
        q{If this is acceptable, then you can dist it by using},
        q{the --no-strict-releasealiases argument (--nsra is an alias):},
        qq{$mp_path/1.0},
    ],
);

$test->efs_test_command(
    command => qq{efs dist release $mp_name dependent --nsra --global},
    expect  => [ dist_release_output(q{dependent}), ],
);

update_releases(q{dependent});
checknolock_releases(q{dependent});

# Teardown everything...
setup_dependencies( \@depends, q{undef} );
destroy_releasealiases(@releasealiases);
destroy_releases(@releases);

my @teardown = ( qq{efs destroy project $mp_name} );

foreach my $teardown (@teardown) {
    $test->efs_test_command(
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;

sub setup_dependencies {

    my ( $depends, $type ) = @_;

    foreach my $fromto (@{$depends}) {
        my ( $from, $to ) = @{$fromto};
        $test->efs_test_command(
            command => qq{efs $type dependency $mp_name $from $mp_name $to runtime},
        );
    }

    return;

}

sub create_releases {
    foreach my $release (@_) {
        $test->efs_test_command(
            command => qq{efs create release $mp_name $release},
        );
        $test->efs_test_command(
            command => qq{efs create install $mp_name $release common},
        );
    }
    return;
}

sub create_releasealiases {
    foreach my $fromto (@_) {
        my ( $from, $to ) = @{$fromto};
        $test->efs_test_command(
            command => qq{efs create releasealias $mp_name $from $to},
        );
    }
    return;
}

sub dist_releasealiases {
    foreach my $fromto (@_) {
        my ( $from, $to ) = @{$fromto};
        $test->efs_test_command(
            command => qq{efs dist releasealias $mp_name $from --global},
        );
    }
    return;
}

sub destroy_releasealiases {
    foreach my $fromto (@_) {
        my ( $from, $to ) = @{$fromto};
        $test->efs_test_command(
            command => qq{efs destroy releasealias $mp_name $from},
        );
    }
    return;
}

sub update_releases {
    foreach my $release (@_) {
        $test->efs_test_command(
            command => qq{touch /efs/dev/$mp_path/$release/install/common/foo},
        );
    }
    return;
}

sub checknolock_releases {
    foreach my $release (@_) {
        $test->efs_test_command(
            command => qq{efs checkpoint release $mp_name $release -nolock},
        );
    }
    return;
}

sub dist_releases {
    foreach my $release (@_) {
        $test->efs_test_command(
            command => qq{efs dist release $mp_name $release --global},
        );
    }
    return;
}

sub destroy_releases {
    foreach my $release (@_) {

        my $stage = q{rejected};

        if ( grep { $release eq $_ } qw( d e ) ) {
            $stage = q{deprecated};
        }

        $test->efs_test_command(
            command => qq{efs setstage release $mp_name $release $stage},
        );
        $test->efs_test_command(
            command => qq{efs destroy release $mp_name $release},
        );
        $test->efs_test_command(
            run_as  => q{efsops},
            command => qq{efs purge release $mp_name $release -now},
        );

    }
    return;
}

sub dist_release_output {

    my $release = shift;

    my @output = (
        q{Dev environment diststate:},
        qr{Install \s+ $ldc_shortname \s+ $rdc_shortname}xms,
        qr{^=+}msx,
        qr{common \s+ \w \s+ \w}xms,
        qr{^-+}msx,
        q{Legend:},
        q{D: Item is disted and up to date},
        q{P: Item is disted but needs update},
        q{X: Item has not been disted},
        qq{Validating the checkpoint state for release '$mp_path/$release'.},
    );

    if ( -l qq{/efs/dist/$mp_path/$release} ) {
        push @output,
            qq{Removed symlink $ldc_name:/efs/dist/$mp_path/$release};
    }

    push @output, (
        qq{Distributing $mp_path/$release from $ldc_name to $ldc_name},
        q{Distribution completed:},
        qq{Target Path = $ldc_name:/efs/dist/$mp_path/$release/common},
        q{Transfered 1 of 2 total files, 0 of 0 total bytes},
        q{Paths updated:},
        q{foo},
        qr{Elapsed \s+ time \s+ = \s+ \d+ \s+ secs}msx,
        qq{Successfully distributed $mp_path/$release/common to $ldc_name},
        qq{Successfully distributed the following installs to cell $ldc_name:},
        qq{$mp_path/$release/common},
        qq{Successfully distributed all installs to $ldc_name},
        qq{Distributing $mp_path/$release from $ldc_name to $rdc_name},
        q{Distribution completed:},
        qq{Target Path = $rdc_name:/efs/dist/$mp_path/$release/common},
        q{Transfered 1 of 2 total files, 0 of 0 total bytes},
        q{Paths updated:},
        q{foo},
        qr{Elapsed \s+ time \s+ = \s+ \d+ \s+ secs}msx,
        qq{Successfully distributed $mp_path/$release/common to $rdc_name},
        qq{Successfully distributed the following installs to cell $rdc_name:},
        qq{$mp_path/$release/common},
        qq{Successfully distributed all installs to $rdc_name},
        q{Successfully distributed release to all cells},
    );

    return @output;

}

sub dist_releasealias_output {

    my ( $from, $to ) = @_;

    my $releasealias = $from;
    my $target       = $to;

    return (
        qq{Releasealias target: $target},
        q{Dev environment diststate:},
        qr{Cell \s+ Target}xms,
        qr{^=+}msx,
        qr{$rdc_shortname \s+ ---}xms,
        qr{^-+}msx,
        q{Legend:},
        q{NNN: Releasealias is disted but target is not up to date},
        q{---: Releasealias is missing from cell},
        qq{Created symlink $rdc_name:/efs/dist/$mp_path/$releasealias -> $target},
        qq{Distributed releasealias $mp_path/$releasealias -> $target to $rdc_name},
        qq{Successfully distributed releasealias $mp_path/$releasealias to all cells},
    );

}
