#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 17;

use blib;
use EFS;

use lib q{test/lib};
use EFS::Test;

my $test = EFS::Test->new;
my $config = $test->get_config;

my $username  = $test->get_primary_username;
my $groupname = $test->get_primary_groupname;

my $sshcmd = $config->get_value( q{Global}, q{sshcmd} );

my $ldc_name      = $config->get_value( q{LocalDevCell}, q{name} );
my $ldc_shortname = $config->get_value( q{LocalDevCell}, q{shortname} );
my $ldc_efsserver = $config->get_value( q{LocalDevCell}, q{efsserver} );

my $rdc_name      = $config->get_value( q{RemoteDevCell}, q{name} );
my $rdc_shortname = $config->get_value( q{RemoteDevCell}, q{shortname} );
my $rdc_efsserver = $config->get_value( q{RemoteDevCell}, q{efsserver} );

my $lpc_name      = $config->get_value( q{LocalProdCell}, q{name} );

my $root_domain = $config->get_value( q{Global}, q{root_domain} );
my $ownerlist   = join q{@}, $username, $root_domain;

my $fileqtree_rs = EFS::DBI->get_resultset( q{Fileqtree} );

my $ldc_qtree = $fileqtree_rs->retrieve_bynames_assert(
    cell       => $ldc_name,
    fileserver => $config->get_value( q{LocalDevCell}, q{fileserver} ),
    fileshare  => $config->get_value( q{LocalDevCell}, q{devshare} ),
    name       => q{efs_qtree},
);
my $local_admin_path = $ldc_qtree->admin_path;

my $rdc_qtree = $fileqtree_rs->retrieve_bynames_assert(
    cell       => $rdc_name,
    fileserver => $config->get_value( q{RemoteDevCell}, q{fileserver} ),
    fileshare  => $config->get_value( q{RemoteDevCell}, q{devshare} ),
    name       => q{efs_qtree},
);
my $remote_admin_path = $rdc_qtree->admin_path;

my @setup = (
    qq{efs create metaproj testns $username $groupname $ownerlist},
    qq{efs create project testns project-a $ldc_name},
    qq{efs create project testns project-b $rdc_name},
);

foreach my $setup (@setup) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs rebuild namespace dist},
    not     => 1,
    expect  => q{The only supported namespaces are 'dev' and 'home'},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs rebuild namespace dev --cells $lpc_name},
    not     => 1,
    expect  => q{No dev cells match the specified criteria},
);

my $history_rs = EFS::DBI->get_resultset( q{History} );
my $history    = $history_rs->create({
    command    => q{move project},
    request    => q{move project x.yyy a.bbb},
    authuser   => q{efsops},
    efshost    => $ldc_efsserver,
    efsversion => q{test},
    pid        => $PID,
});

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs rebuild namespace dev --cells $ldc_name},
    not     => 1,
    expect  => [
        q{The following move commands are still active:},
        qr{Id \s+ Authuser \s+ Efshost \s+ Pid \s+ Request}msx,
        qr{ =+ }msx,
        qr{ $PID \s+ move \s+ project \s+ x.yyy \s+ a.bbb }msx,
        q{This command may not be used while active moves are in progress},
    ],
);

$history->delete;


$test->efs_test_command(
    run_as  => q{root},
    command => qq{unlink $local_admin_path/global/testns/project-a},
);

$test->efs_test_command(
    run_as  => q{root},
    command => qq{mkdir $local_admin_path/global/bogusmeta},
);

$test->efs_test_command(
    run_as  => q{root},
    command => qq{ln -s /bogus/symlink $local_admin_path/global/bogusmeta/bogusproj},
);

$test->efs_test_command(
    run_as  => q{root},
    command => qq{$sshcmd $rdc_efsserver rm -f $remote_admin_path/global/testns/project-b},
);

$test->efs_test_command(
    run_as  => q{root},
    command => qq{$sshcmd $rdc_efsserver mkdir -p $remote_admin_path/global/bogusmeta},
);

$test->efs_test_command(
    run_as  => q{root},
    command => qq{$sshcmd $rdc_efsserver rm -f $remote_admin_path/global/bogusmeta/bogusproj},
);

$test->efs_test_command(
    run_as  => q{root},
    command => qq{$sshcmd $rdc_efsserver ln -s /bogus/symlink $remote_admin_path/global/bogusmeta/bogusproj},
);

my $symlink_a = q{/efs/dev/testns/project-a};
my $symlink_b = q{/efs/dev/testns/project-b};

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs rebuild namespace dev},
    expect  => [
        qr{Created \s+ symlink \s+ $ldc_name:/efs/dev/testns/project-a }msx,
        qq{Removed symlink $ldc_name:/efs/dev/bogusmeta/bogusproj},
        qq{Removed directory $ldc_name:/efs/dev/bogusmeta},
        qq{Change summary for the dev namespace in $ldc_name:},
        qr{ links \s+ added \s+ or \s+ changed \s+ => \s+ 1 }msx,
        qr{ dirs \s+ added \s+ => \s+ 0 }msx,
        qr{ links \s+ removed \s+ => \s+ 1 }msx,
        qr{ dirs \s+ removed \s+ => \s+ 1 }msx,
        qr{Created \s+ symlink \s+ $rdc_name:/efs/dev/testns/project-b }msx,
        qq{Removed symlink $rdc_name:/efs/dev/bogusmeta/bogusproj},
        qq{Removed directory $rdc_name:/efs/dev/bogusmeta},
        qq{Change summary for the dev namespace in $rdc_name:},
        qr{ links \s+ added \s+ or \s+ changed \s+ => \s+ 1 }msx,
        qr{ dirs \s+ added \s+ => \s+ 0 }msx,
        qr{ links \s+ removed \s+ => \s+ 1 }msx,
        qr{ dirs \s+ removed \s+ => \s+ 1 }msx,
    ],
);

my @teardown = (
    q{efs destroy project testns project-a},
    q{efs destroy project testns project-b},
    q{efs destroy metaproj testns},
);

foreach my $teardown ( @teardown ) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

